from __future__ import absolute_import
from __future__ import unicode_literals

import requests
from minio import Minio
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.middlewarehandler import MinioConfigHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, status_me


class MinioController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: MinioConfigHandler, template: str):
        super(MinioController, self).__init__(result, handler, template)
        self.minio_cfg = MinioConfigHandler().get_config_with_check_result()
        self.minio_nginx_tmp = path_join("/tmp", "nginx-minio.conf")
        self.minio_nginx_template = path_join(self.template, "nginx-minio.tmpl")
        self.minio_nginx_file = path_join(self.deploy_dir, "nginx/http/nginx-minio.conf")
        self.minio_cfg["minio"]["ipaddress"] = self.handler.cfg["ipaddress"]

    def _proxy_on_nginx(self):
        log.info("渲染 Nginx 代理配置文件")
        FileTemplate(self.minio_cfg, self.minio_nginx_template, self.minio_nginx_tmp).fill()
        self.update_nginx_config()
        self.generate_docker_compose_file(self.minio_cfg)

    def _create_bucket(self, bucket: str):
        log.info("创建 bucket")
        try:
            self.endpoint = "{}:{}".format(self.minio_cfg["minio"]["lb"], self.minio_cfg["minio"]["nginx_proxy_port"])
            cli = Minio(
                self.endpoint,
                self.minio_cfg["minio"]["username"],
                self.minio_cfg["minio"]["password"],
                secure=False
            )
            if not cli.bucket_exists(bucket):
                cli.make_bucket(bucket)
                return True
        except Exception as e:
            log.error(e)
            log.error("Minio 创建bucket失败")
            exit(2)

    def check(self):
        super().wait_for_service_start()
        for ip in self.master_server:
            try:
                log.info("检查 {IP}:{PORT} 上的 Minio 健康性...".format(IP=ip.ipaddress, PORT=self.minio_cfg["minio"]["port"]))
                requests.get(
                    "http://{IP}:{PORT}/minio/health/live".format(
                        IP=ip.ipaddress, PORT=self.minio_cfg["minio"]["port"]
                    )
                )
                log.info("Minio 连接检查通过")
            except Exception as e:
                log.error(e)
                log.error("{IP}:{PORT} 上的 Minio 服务异常".format(IP=ip.ipaddress,PORT=self.minio_cfg["minio"]["port"]))
                exit(2)

    @status_me("middleware")
    def deploy_minio(self):
        if self.check_is_deploy(self.minio_cfg):
            self._proxy_on_nginx()
            self.send_docker_compose_file()
            self.start()
            self.check()
            self._create_bucket("mysql-backup")

    def deploy(self):
        self.deploy_minio()



#!/usr/bin/env python
# coding=utf-8
from __future__ import print_function
from jinja2 import Environment, FileSystemLoader
import jinja2
import sys
import toml
import json
import yaml
import io
import re
import os

if sys.version > '3':
    PY3 = True
    import configparser
else:
    PY3 = False
    from io import open
    import ConfigParser as configparser

class Conf(object):
    _conf_file = []

    @classmethod
    def add_conf(cls, conf):
        if os.path.isfile(conf):
            cls._conf_file.append(conf)
        elif os.path.isdir(conf):
            for i in os.listdir(conf):
                if re.match(r'.*\.(yaml|yml)', i):
                    cls._conf_file.append(os.path.join(conf, i))

    @classmethod
    def init(cls):
        setattr(cls, 'all_property', {})
        conf_dict = {}
        for c in cls._conf_file:
            if os.path.exists(c):
                conf_dict.update(yaml.load(open(c), Loader=yaml.Loader))
        for k, v in conf_dict.items():
            setattr(cls, k, v)
            cls.all_property[k] = v

    @classmethod
    def reset(cls):
        cls._conf_file = []
        cls.all_property = {}

    @classmethod
    def reload(cls, conf_file):
        cls._conf_file = []
        cls.all_property = {}
        cls.add_conf(conf_file)
        cls.init()

    @classmethod
    def debug(cls):
        return cls._conf_file


def fill_config_util(template_path, target, data, default_key=True):
    """渲染模板

        Args:
            template_path: 配置文件模板
            target: 目标文件
            data: 数据（字典）
            default_key: 是否处理默认参数
        Returns:
            None
        """
    FillConfig.load_template(template_path).generate_config(target, data, default_key)


class FillConfig:

    def __init__(self, template_path):
        self.template_path = template_path
        self.loader = FileSystemLoader(template_path)
        self.template_env = Environment(loader=self.loader, undefined=jinja2.StrictUndefined)
        self.format = ('json', 'toml', 'yaml', 'ini')
        self.template_env.filters['regex_replace'] = self._regex_replace

    def _regex_replace(self, s, find, replace):
        return re.sub(find, replace, s)

    @classmethod
    def load_template(cls, template_path):
        return FillConfig(template_path)

    def list_templates(self):
        """获取模板

        Args:
            None
        Returns:
            list
        """
        return self.loader.list_templates()

    def _dir_generate_config(self, target_path, data):
        if not os.path.exists(target_path):
            os.makedirs(target_path)

        template_names = self.list_templates()
        for template_name in template_names:
            target_file_path = os.path.join(target_path, template_name)
            template = self.template_env.get_template(template_name)
            content = template.render(data)
            with open(target_file_path, "w", encoding='UTF-8') as f:
                r, t = self.check_format(content)
                if r:
                    f.write(content)

    def _file_generate_config(self, target_file_path, data):
        content = self.template_env.get_template('').render(data)
        if not os.path.exists(os.path.dirname(target_file_path)):
            os.makedirs(os.path.dirname(target_file_path))
        with open(target_file_path, "w", encoding='UTF-8') as f:
            f.write(content)

    def _default_key_handler(self, data, separator='Defaule'):
        return DefaultKeyHandler(data).fill_config_dict()

    def generate_config(self, target_file, data, config_has_default_key=False):
        """生成模板

        Args:
            template_name: 模板名称
            target_file: 目标文件
        Returns:
            None
        """
        if config_has_default_key:
            data = self._default_key_handler(data)

        if os.path.isdir(self.template_path):
            if os.path.isdir(target_file) or not os.path.exists(target_file):
                self._dir_generate_config(target_file, data)
            else:
                return
        elif os.path.isfile(self.template_path):
            if os.path.exists(target_file) and os.path.isdir(target_file):
                return
            self._file_generate_config(target_file, data)

    def _load_test(self, content, format):
        try:
            if format == 'yaml':
                yaml.load(content)
            elif format == 'ini':
                conf = configparser.ConfigParser()
                if PY3 is False:
                    conf.readfp(io.StringIO(content))
                else:
                    conf.read_string(content)
            elif format == 'json':
                c = re.sub(r'''/\*.*\*/.*''', '', content)
                json.loads(c)
            elif format == 'toml':
                toml.loads(content)
        except Exception:
            return False
        return True

    def check_format(self, content):
        for fm in self.format:
            if self._load_test(content, fm):
                return True, fm
        return False, None


class DefaultKeyHandler(object):

    def __init__(self, config_has_default_key, key_separator='Default'):
        self.config = config_has_default_key
        self.key_separator = key_separator

    def get_default_key(self, separator='Default'):
        """获取默认值

        Args:
            config: 配置文件列表
            split: 分隔符
        Returns:
            list
        """
        return list(filter(lambda x: x if re.search(separator, x) else None, self.config.keys()))

    def fill_config_dict(self):
        default_keys = self.get_default_key()

        for default_key in default_keys:
            default_key_prefix = default_key.split(self.key_separator)[0]
            other_keys = self.get_default_key(default_key_prefix)
            for key in self.config[default_key]:
                for other_key in other_keys:
                    if self.config[other_key] is None:
                        self.config[other_key] = {}
                    if key not in self.config[other_key]:
                        self.config[other_key][key] = self.config[default_key][key]
        return self.config



from __future__ import absolute_import
from __future__ import unicode_literals

import os
from jinja2 import Environment, FileSystemLoader, StrictUndefined
from laipvt.helper.exception import UtilsError
from laipvt.sysutil.util import path_join, log

class Template(object):
    def __init__(self, data: dict, file_path: str, file_dest: str):
        self.file_path = file_path
        self.file_dest = file_dest
        self.data = data

    def template_handler(self, src: str) -> str:
        pass

    def _fill(self, src, dest):
        log.debug("渲染模板: {}到目录: {}".format(src, dest))
        dest_dir = os.path.dirname(dest)
        if not os.path.exists(dest_dir):
            os.makedirs(dest_dir)
        content = self.template_handler(src)
        with open(dest, 'w+', encoding='utf-8') as fp:
            fp.write(content)

    def fill(self):
        if os.path.isfile(self.file_path):
            file_path = self.file_path
            file_dest = self.file_dest
            self._fill(file_path, file_dest)
        elif os.path.isdir(self.file_path):
            if not os.path.exists(self.file_dest):
                os.makedirs(self.file_dest)
            self.fill_all(self.file_path, self.file_dest)
        else:
            log.error("self.file_path: {}, self.file_dest: ".format(self.file_path, self.file_dest))
            raise UtilsError("Template.fill, 错误信息: 目录类型不正确或目录不存在") from None

    def fill_all(self, src, dest):
        try:
            for root, dirs, files in os.walk(src):
                for file_name in files:
                    file_path = path_join(root, file_name)
                    if len(root) == len(src):
                        file_dest = path_join(dest, file_name)
                        log.info("渲染模板到{}".format(path_join(dest, file_name)))
                    else:
                        new_path = root.replace("{}/".format(src), "")
                        while True:
                            if new_path.startswith("/"):
                                new_path = new_path[1:]
                            else:
                                break
                        file_dest = path_join(dest, new_path, file_name)
                        log.debug("找到子目录下的文件")
                        log.info("渲染模板到{}".format(file_dest))
                    self._fill(file_path, file_dest)
        except Exception as e:
            raise UtilsError("FileTemplate.fill_all, 错误信息: {}".format(e)) from None

    def run(self):
        pass


class FileTemplate(Template):
    def __init__(self, data, file_path, file_dest):
        super().__init__(data, file_path, file_dest)

    def template_handler(self, src):
        try:
            loader = FileSystemLoader(src)
            env = Environment(loader=loader, undefined=StrictUndefined)
            return env.get_template('').render(self.data)

        except Exception as e:
            raise UtilsError("FileTemplate.template_handler, 错误信息: {}".format(e)) from None


class ConfigMapTemplate(Template):
    def __init__(self):
        super().__init__()



from os import path as os_path
from setuptools import setup

import laipvt

this_directory = os_path.abspath(os_path.dirname(__file__))

def read_file(filename):
    with open(os_path.join(this_directory, filename), encoding='utf-8') as f:
        long_description = f.read()
    return long_description

def read_requirements(filename):
    return [line.strip() for line in read_file(filename).splitlines()
            if not line.startswith('#')]

setup(
    name='laipvt',
    entry_points={
        'console_scripts': ['laipvt=laipvt.cli.main:main'],
    },
    python_requires='>=3.6.0',
    version=laipvt.__version__,
    description="laiye private deploy basement",
    long_description="laiye private deploy basement",
    long_description_content_type="text/markdown",
    author="ye",
    author_email='liye@laiye.com',
    url='https://laiye.com',
    packages= [
        "laipvt",
        "laipvt.controller",
        "laipvt.controller.kubernetes",
        "laipvt.controller.middleware",
        "laipvt.controller.service",
        "laipvt.handler",
        "laipvt.helper",
        "laipvt.interface",
        "laipvt.model",
        "laipvt.sysutil",
        "laipvt.cli",
        "upgrade"
    ],
    install_requires=read_requirements('requirements.txt'),
    include_package_data=True,
    license="MIT",
    keywords=['laipvt'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)


from __future__ import absolute_import
from __future__ import unicode_literals
import os
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.middlewarehandler import NginxConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, status_me, ssh_obj
from laipvt.model.cmd import ComposeModel
from laipvt.sysutil.gvalue import CERTS_PATH



class NginxController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: NginxConfigHandler, template: str):
        super(NginxController, self).__init__(result, handler, template)

        self.nginx_conf_tmp = path_join("/tmp", "nginx.conf")
        self.nginx_conf_template = path_join(self.template, "config.tmpl")
        self.nginx_conf_file = path_join(self.base_dir, "nginx.conf")
        self.apiserver_conf_tmp = path_join("/tmp", "apiserver.conf")
        self.apiserver_conf_template = path_join(self.template, "tcp/apiserver.conf")
        self.apiserver_conf_file = path_join(self.base_dir, "tcp/apiserver.conf")
        self.apiserver_cluster_conf_template = path_join(self.template, "tcp/apiserver_cluster.conf")
        self.nginx_cfg = NginxConfigHandler().get_config_with_check_result()
        self.nginx_cfg["nginx"]["ipaddress"] = self.handler.cfg["ipaddress"]
        self.nginx_cfg.update(self.check_result.__dict__)
        # if self.nginx_cfg["config"]["deploy_https"]:
        #     if self.nginx_cfg["config"]["self_signed_ca"]:
        #         self.nginx_cfg["config"]["cert_path"] = os.path.dirname(SERVER_SECURE_KEY)
        #     else:
        #         self.nginx_cfg["config"]["cert_path"] = os.path.dirname(self.nginx_cfg["config"]["ca_key_path"])

    def _generic_config(self):
        if self.nginx_cfg["config"]["deploy_https"]:
            if self.nginx_cfg["config"]["self_signed_ca"]:
                self.nginx_cfg["config"]["cert_path"] = CERTS_PATH
                self._send_https_ca_file()
            else:
                self.nginx_cfg["config"]["cert_path"] = os.path.dirname(self.nginx_cfg["config"]["ca_key_path"])

        log.info("渲染nginx配置文件")
        for num_id in range(len(self.all_server)):
            FileTemplate(self.nginx_cfg, self.nginx_conf_template, self.nginx_conf_tmp).fill()
            FileTemplate(self.nginx_cfg, self.apiserver_conf_template, self.apiserver_conf_tmp).fill()
            self.send_config_file(self.all_server[num_id], self.nginx_conf_tmp, self.nginx_conf_file)
            self.send_config_file(self.all_server[num_id], self.apiserver_conf_tmp, self.apiserver_conf_file)
        self.generate_docker_compose_file(self.nginx_cfg)

    def _send_docker_compose_file(self):
        for server in self.all_server:
            log.info("分发%s docker-compose配置文件至 %s" % (self.middleware_name, server.ipaddress))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                ssh_cli.put(self.docker_compose_file_tmp, self.docker_compose_file)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def _send_https_ca_file(self):
        for server in self.all_server:
            log.info("分发%s https证书文件至 %s" % (self.middleware_name, server.ipaddress))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                ssh_cli.put("/tmp/certs", CERTS_PATH)

            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def renew_apiserver_config(self):
        log.info("将k8s master节点渲染到apiserver配置文件中")
        for num_id in range(len(self.all_server)):
            FileTemplate(self.nginx_cfg, self.apiserver_cluster_conf_template, self.apiserver_conf_tmp).fill()
            self.send_config_file(self.all_server[num_id], self.apiserver_conf_tmp, self.apiserver_conf_file)
            self._restart()

    def _check(self):
        super().wait_for_service_start()
        port = self.nginx_cfg["nginx"]["k8s_proxy_port"]
        for server in self.master_server:
            res = self.check_port(server.ipaddress, port)
            if res:
                log.info("机器:{} Nginx代理端口:{}访问正常".format(server.ipaddress, port))
            else:
                log.error("机器:{} Nginx代理端口:{}访问异常".format(server.ipaddress, port))
                exit(2)

    def _start(self):
        compose_cmd = ComposeModel(self.docker_compose_file)
        for server in self.all_server:
            log.info("启动 %s %s服务" % (server.ipaddress, self.middleware_name))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                res = ssh_cli.run_cmd(compose_cmd.up())
                if res["code"] != 0:
                    log.error("启动 %s: %s服务失败.错误原因: %s %s" % (server.ipaddress, self.middleware_name, res["stderr"], res["stdout"]))
                    exit(2)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def _restart(self):
        compose_cmd = ComposeModel(self.docker_compose_file)
        for server in self.all_server:
            log.info("启动 %s %s服务" % (server.ipaddress, self.middleware_name))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                res = ssh_cli.run_cmd(compose_cmd.restart())
                if res["code"] != 0:
                    log.error("启动 %s: %s服务失败.错误原因: %s %s" %
                              (server.ipaddress, self.middleware_name, res["stderr"], res["stdout"]))
                    exit(2)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    @status_me("basesystem")
    def install_nginx(self):
        self._generic_config()
        self._send_docker_compose_file()
        self._start()
        self._check()



from laipvt.model.cmd import DockerImageModel
from laipvt.model.harbor import HarborModel
from laipvt.sysutil.util import path_join, log
import os

def ImportImage(project, path, user, passwd, addr):
    harbor = HarborModel(addr, user, passwd)
    if project not in harbor.list_project():
        harbor.create_project(project)
    image_path = path_join(path, "images")
    for image_name in os.listdir(image_path):
        img_p = path_join(image_path, image_name)
        log.info("将镜像push到私有仓库: {}".format(img_p))
        # print(image_path)
        docker = DockerImageModel(image=img_p, project=project, repo=addr)
        docker.run()


from __future__ import absolute_import
from __future__ import unicode_literals
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.middlewarehandler import MysqlConfigHandler, MinioConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import walk_sql_path, path_join, log, ssh_obj, status_me
from laipvt.model.sql import SqlModule
from laipvt.model.server import ServerModel
from laipvt.sysutil.conf import YamlConfig
from laipvt.sysutil.gvalue import LAIPVT_BASE_DIR


class MysqlController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: MysqlConfigHandler, template: str):
        super(MysqlController, self).__init__(result, handler, template)
        self.mysql_cfg = YamlConfig(path_join(LAIPVT_BASE_DIR, "middleware"), suffix="yaml").read_dir()
        self.mysql_cfg["mysql"] = MysqlConfigHandler().get_config_with_check_result()["mysql"]
        self.user = self.mysql_cfg["mysql"]["username"]
        self.password = self.mysql_cfg["mysql"]["password"]
        self.port = int(self.mysql_cfg["mysql"]["port"])
        self.master_host = self.master_server[0].ipaddress
        self.mysql_nginx_tmp = path_join("/tmp", "nginx-mysql.conf")
        self.proxysql_conf_tmp = path_join("/tmp", "proxysql.cnf")
        self.proxysql_conf_template = path_join(self.template, "proxysql.cnf")
        self.mysql_nginx_template = path_join(self.template, "nginx-mysql.tmpl")
        self.proxysql_conf_file = path_join(self.base_dir, "conf", "proxysql.cnf")
        self.template_file_name = ("mysql_master.cnf", "mysql_slave1.cnf", "mysql_slave2.cnf")
        self.mysql_config_file = path_join(self.base_dir, "conf", "{}.conf".format(self.middleware_name))
        self.template_sql = path_join(self.template, "mgr_init")
        self.template_sql_name = ("fill_slave_join_mgr.sql", "fill_create_mgr.sql", "fill_proxysql_join_mgr.sql")
        self.mysql_service_file_template = path_join(self.template, "mysql_service.tmpl")
        self.mysql_service_file_dest = path_join("/tmp", "mysql_service.yaml")
        self.mysql_service_file_remote = path_join(self.base_dir, "svc", "mysql_service.yaml")
        if self.mysql_cfg["mysql"]["is_deploy"]:
            self.mysql_cfg["mysql"]["ipaddress"] = self.handler.cfg["ipaddress"]
        self.mysql_cfg["minio"]["ipaddress"] = self.handler.cfg["ipaddress"]
        self.mysql_cfg["mysql"]["data_dir"] = path_join(self.base_dir, "data/mysql")
        self.backup_conf_template = path_join(self.template, "backup.cnf")
        self.backup_conf_tmp = path_join("/tmp", "backup.cnf")
        self.backup_conf_file = path_join(self.base_dir, "conf", "backup.conf")
        self.backup_sh_template = path_join(self.template, "backup.sh")
        self.backup_sh_file = path_join(self.base_dir, "backup.sh")

    def gen_proxy_sql_config(self):
        log.info("渲染分发Proxysql配置文件")
        server = ServerModel(self.master_server)
        try:
            FileTemplate(self.mysql_cfg, self.proxysql_conf_template, self.proxysql_conf_tmp).fill()
            server.send_file(self.proxysql_conf_tmp, self.proxysql_conf_file)
        except Exception as e:
            log.error(e)
            exit()
        finally:
            server.close()

    def _fill_init_sql(self):
        if self.mysql_cfg["is_standalone"]:
            return
        for file in self.template_sql_name:
            sql_file = path_join(self.template_sql, file)
            FileTemplate(self.mysql_cfg, sql_file, sql_file.replace("fill_", "")).fill()
        self.gen_proxy_sql_config()
        self._proxy_on_nginx()

    def _generic_config(self):
        log.info("渲染Mysql配置文件")
        if len(self.master_server) == 1:
            log.info("使用单机模式部署")
            self.mysql_cfg["is_standalone"] = True
        elif len(self.master_server) == 3:
            log.info("使用集群模式部署")
            self.mysql_cfg["is_standalone"] = False
        else:
            log.info("集群模式需要3台主机,请检查前置检查配置是否正确")
            exit(2)
        for num_id in range(len(self.master_server)):
            self.mysql_cfg["server_id"] = 100 + num_id
            src = path_join(self.template, self.template_file_name[num_id])
            dest = path_join("/tmp", self.template_file_name[num_id])
            FileTemplate(self.mysql_cfg, src, dest).fill()
            self.mysql_cfg["mysql"]["ip"] = self.master_server[num_id].ipaddress
            FileTemplate(self.mysql_cfg, self.backup_conf_template, self.backup_conf_tmp).fill()
            self.send_config_file(self.master_server[num_id], dest, self.mysql_config_file)
            self.send_config_file(self.master_server[num_id], self.backup_conf_tmp, self.backup_conf_file)
            self.send_config_file(self.master_server[num_id], self.backup_sh_template, self.backup_sh_file)
        self.generate_docker_compose_file(self.mysql_cfg)

    def _proxy_on_nginx(self):
        log.info("渲染Mysql代理配置文件")
        for num_id in range(len(self.master_server)):
            FileTemplate(self.mysql_cfg, self.mysql_nginx_template, self.mysql_nginx_tmp).fill()
        self.update_nginx_config()

    def _create_mgr_cluster(self):
        """
        构建mysqlmgr集群
        :return:
        """
        super().wait_for_service_start()
        if self.mysql_cfg["is_standalone"]:
            return True
        log.info("构建Proxysql MysqlMgr 集群")
        sql = SqlModule(host=self.master_host, port=self.port, user=self.user, passwd=self.password)
        sql.import_from_file(path_join(self.template, "mgr_init/create_mgr.sql"), file_eof=";" )
        sql.import_from_file(path_join(self.template, "mgr_init/view.sql"), file_eof="$$")

        log.info("Slave 节点加入mgr集群")
        for server in self.master_server[1:]:
            sql = SqlModule(host=server.ipaddress, port=self.port, user=self.user, passwd=self.password)
            sql.import_from_file(path_join(self.template, "mgr_init/slave_join_mgr.sql"), file_eof=";")

        log.info("Proxysql 添加mgr节点")
        sql = SqlModule(host=self.master_host, port=6032, user="cluster", passwd="123456", connect_timeout=5)
        sql.import_from_file(path_join(self.template, "mgr_init/proxysql_join_mgr.sql"), file_eof=";")

    def _check(self):
        super().wait_for_service_start()
        for server in self.master_server:
            if not self.mysql_cfg["is_standalone"]:
                self.port = int(self.mysql_cfg["mysql"]["proxysql_port"])
            try:
                log.info("检查 {IP}:{PORT} 上的 MySQL 健康性".format(IP=server.ipaddress, PORT=self.port))
                SqlModule(
                    host=server.ipaddress, port=self.port,
                    user=self.user, passwd=self.password,
                    db="test", charset='utf8'
                )
                self._read_write_test(server.ipaddress, self.port)
            except Exception as e:
                log.error(e)
                log.error("{IP}:{PORT} 上的 MySQL 服务异常".format(IP=server.ipaddress, PORT=self.port))
                exit(2)
        # if not self.handler.cfg["is_standalone"]:
        #     self._read_write_test(self.master_host, self.handler.cfg["mysql_load"]["nginx_proxy_port"])

    def _read_write_test(self, Ipaddress, Port):
        log.info("测试验证Mysql数据库读写")
        try:
            sql = SqlModule(host=Ipaddress, port=Port, user=self.user, passwd=self.password)
            sql.insert_sql("create table test.student (id int(10),name char(100),primary key (ID));")
            sql.insert_sql("insert into test.student (id,name)values(1,'Reading and writing tests');")
            sql.select("select * from test.student;")
            sql.insert_sql("drop table test.student;")
            log.info("Mysql {IP}:{PORT} 读写测试成功...".format(IP=Ipaddress, PORT=Port))

        except Exception as e:
            log.info("Mysql {IP}:{PORT} 读写测试失败...".format(IP=Ipaddress, PORT=Port))
            log.error(e)
            exit(2)

    def init(self, path):
        log.info(path)
        db_info = walk_sql_path(path)
        for db_name, sql_files in db_info.items():
            sql = SqlModule(host=self.master_host, port=self.port, user=self.user, passwd=self.password)
            create_db = "create database If Not Exists {db_name} DEFAULT CHARSET utf8mb4 COLLATE utf8mb4_general_ci".format(
                db_name=db_name
            )
            sql.insert_sql(create_db)
            for sql_file in sql_files:
                sql = SqlModule(host=self.master_host, port=self.port, user=self.user, passwd=self.password, db=db_name)
                sql.import_from_file(sql_file, file_eof=";\n")

    def run_backup_crontab(self):
        log.info("配置Mysql 备份定时任务")
        self.crontab_task = "01 00 * * * sudo docker run --rm  " \
                            "-v {}:/mysql-backup  -v {}:/var/lib/mysql" \
                            " -v {}:/backup.sh -v {}:/backup.conf  {}:{}/middleware/mysql_xtrabackup:5.7.32 /bin/bash "\
                            "-c '/backup.sh /backup.conf /mysql-backup/backup.log &>> /mysql-backup/backup.log'"
        self.crontab_task = self.crontab_task.format(
            path_join(self.base_dir, "mysql-backup"),
            path_join(self.base_dir, "data/mysql"),
            self.backup_sh_file,
            self.backup_conf_file,
            self.mysql_cfg["mysql"]["harbor_ipaddress"],
            self.mysql_cfg["mysql"]["nginx_harbor_proxy_port"]
            )
        self.chmod_backupsh = "chmod +x {}".format(self.backup_sh_file)

        for server in self.master_server:
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            cmd_list = [self.chmod_backupsh,
                        "systemctl restart crond",
                        "echo \"{}\" >> /var/spool/cron/{}".format(self.crontab_task, server.username)]
            results = ssh_cli.run_cmdlist(cmd_list)
            if results[0]["code"] == 0 and results[1]["code"] == 0:
                log.info("{} Crontab 定时启动并配置成功".format(server.ipaddress))
            else:
                log.error(results[0]["stdout"])
                log.error(results[1]["stdout"])
                log.error("{} 主机 Crontab 配置失败".format(server.ipaddress))
                exit(2)

    def create_mysql_service_kubernetes(self):
        log.info("渲染初始化Mysql Service in Kubernetes")
        server = ServerModel(self.master_server)
        FileTemplate(self.mysql_cfg, self.mysql_service_file_template, self.mysql_service_file_dest).fill()
        server.send_file(self.mysql_service_file_dest, self.mysql_service_file_remote)

        log.info("在kubernetes集群内创建Mysql Service")
        cmd = "kubectl apply -f {}".format(self.mysql_service_file_remote)
        ssh_cli = ssh_obj(ip=self.master_server[0].ipaddress, user=self.master_server[0].username,
                          password=self.master_server[0].password, port=self.master_server[0].port)
        results = ssh_cli.run_cmd(cmd)
        if results["code"] != 0:
            log.error("kubernetes集群内创建Mysql Service失败:{} {}".format(results["stdout"], results["stderr"]))
            exit(2)

    @status_me("middleware")
    def deploy_mysql(self):
        if self.check_is_deploy(self.mysql_cfg):
            self._generic_config()
            self._fill_init_sql()
            self.send_docker_compose_file()
            self.start()
            self.run_backup_crontab()
            self._create_mgr_cluster()
            self._check()
            self.create_mysql_service_kubernetes()

    def deploy(self):
        self.deploy_mysql()



from __future__ import absolute_import
from __future__ import unicode_literals

class ColorPrint(object):
    colors = {
        "red": 31,
        "green": 32
    }

    @classmethod
    def get(cls, msg, color="red"):
        return "\033[1;{}m{}\033[0m".format(cls.colors[color.lower()], msg)

    @classmethod
    def print(cls, msg, color="red"):
        print("\033[1;{}m{}\033[0m".format(cls.colors[color.lower()], msg))


from __future__ import absolute_import
from __future__ import unicode_literals

import re
import time
import requests
from requests.auth import HTTPBasicAuth
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.middlewarehandler import EsConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, status_me


class EsController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: EsConfigHandler, template: str):
        super(EsController, self).__init__(result, handler, template)
        self.es_conf_tmp = path_join("/tmp", "elasticsearch.yml")
        self.es_conf_template = path_join(self.template, "config.tmpl")
        self.es_conf_file = path_join(self.base_dir, "conf", "elasticsearch.yml")
        self.auth_conf_tmp = path_join("/tmp", "readonlyrest.yml")
        self.auth_conf_template = path_join(self.template, "readonlyrest.yml")
        self.auth_conf_file = path_join(self.base_dir, "conf", "readonlyrest.yml")
        self.pligins = path_join(self.template, "plugins")
        self.pligins_tmp = path_join(self.base_dir, "elasticsearch")
        self.es_cfg = EsConfigHandler().get_config_with_check_result()
        self.http_port = self.es_cfg["elasticsearch"]["http_port"]
        self.password = self.es_cfg["elasticsearch"]["password"]
        self.es_cfg["elasticsearch"]["ipaddress"] = self.handler.cfg["ipaddress"]

    def _generic_config(self):
        log.info("渲染 Elasticsearch 认证配置文件")
        for num_id in range(len(self.master_server)):
            self.es_cfg["elasticsearch"]["host_name"] = "saas-es-0%s" % (num_id + 1)
            self.es_cfg["elasticsearch"]["master_node"] = "saas-es-01"
            self.es_cfg["ip"] = self.master_server[num_id].ipaddress
            log.info("渲染 Elasticsearch 配置文件")
            FileTemplate(self.es_cfg, self.es_conf_template, self.es_conf_tmp).fill()
            FileTemplate(self.es_cfg, self.auth_conf_template, self.auth_conf_tmp).fill()

            self.send_config_file(self.master_server[num_id], self.es_conf_tmp, self.es_conf_file)
            self.send_config_file(self.master_server[num_id], self.auth_conf_tmp, self.auth_conf_file)
            self.send_config_file(self.master_server[num_id], self.pligins, self.pligins_tmp)
            self.generate_docker_compose_file(self.es_cfg)

    def _check(self):
        time.sleep(60)
        for i in self.master_server:
            try:
                url = "http://{IP}:{PORT}/_cat/health".format(
                    IP=i.ipaddress, PORT=self.http_port
                )
                log.info("检查 {}:{} 上的 Elasticsearch 健康性".format(i.ipaddress, self.http_port))
                result = requests.get(url,  auth=HTTPBasicAuth('admin', '{}'.format(
                    self.password))).content.decode("utf-8")

                if re.search("green", result):
                    # log.info("Elasticsearch 检查通过")
                    self._read_write_test(i.ipaddress, self.http_port)
                else:
                    log.error("{IP} 上的 Elasticsearch 服务异常".format(IP=i.ipaddress))
                    log.error("错误日志: {}".format(result))
                    exit(2)
            except Exception as e:
                log.error(e)
                log.error("{IP} 上的 Elasticsearch 服务异常".format(IP=i.ipaddress))
                exit()

    def _read_write_test(self, ipaddress, port):
        try:
            headers = {"Content-Type": "application/json"}
            create_index = "http://{IP}:{PORT}/laiye_test/_doc/1".format(IP=ipaddress, PORT=port)
            delete_index = "http://{IP}:{PORT}/laiye_test".format(IP=ipaddress, PORT=port)
            data_dit = {"name": "laiye", "age": "5"}
            auth = HTTPBasicAuth('admin', '{}'.format(self.password))

            requests.post(url=create_index, json=data_dit,  headers=headers, auth=auth)
            log.info("Elasticsearch 测试数据插入成功")
            requests.delete(url=delete_index, auth=auth)
        except Exception as e:
            log.error(e)
            exit(2)

    @status_me("middleware")
    def deploy_es(self):
        if self.check_is_deploy(self.es_cfg):
            self._generic_config()
            self.send_docker_compose_file()
            self.start()
            self._check()

    def deploy(self):
        self.deploy_es()



from __future__ import absolute_import
from __future__ import unicode_literals

import os
from laipvt.sysutil.conf import YamlConfig
from laipvt.sysutil.util import path_join, gen_pass
from laipvt.sysutil.gvalue import LAIPVT_BASE_DIR, CHECK_FILE
from laipvt.sysutil.relation import get_port
from laipvt.sysutil.wraps import check_value
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.helper.exception import UtilsError


class MiddlewareConfigHandler(object):
    def __init__(self, name: str):
        self.permit_keys_base = ("ipaddress", "is_deploy", "lb")
        self.permit_keys = ()
        self.laipvt_base_dir = LAIPVT_BASE_DIR
        self.name = name
        self.local_cfg = path_join(self.laipvt_base_dir, "middleware", "{}.yaml".format(self.name))
        self.cfg = {}
        self.cfg["name"] = self.name
        self.all_cfg = {}

    def set_deploy_type(self):
        try:
            self.cfg["deploy_type"] = "single" if len(self.cfg["ipaddress"]) <= 1 else "cluster"
        except TypeError:
            self.cfg["deploy_type"] = "single"

    @check_value
    def set(self, key, value) -> bool:
        if key in self.permit_keys or key in self.permit_keys_base:
            self.cfg[key] = value
            if key == "ipaddress":
                self.set_deploy_type()
            return True
        return False

    def get(self) -> dict:
        return self.cfg

    def get_value(self, value: str):
        return self.cfg[value]

    def get_proxy_type(self) -> str:
        d = {
            "minio": "http",
            "rabbitmq": "tcp",
            "elasticsearch": "http",
            "nsq": "tcp",
            "dgraph": "tcp",
            "identity": "http",
            "mongo": "tcp",
            "etcd": "tcp",
            "mysql": "tcp",
            "mage": "http"
        }
        return d.get(self.cfg["name"], "http")

    def save(self) -> bool:
        try:
            dict_cfg = {}
            dict_cfg[self.name] = self.cfg
            dumper = YamlConfig(self.local_cfg, dict_cfg)
            dumper.write_file(backup=False)
            return True
        except Exception as e:
            print(e)
            return False

    def load(self) -> dict:
        loder = YamlConfig(self.local_cfg)
        return loder.read_file()

    def load_config_from_file(self):
        if os.path.isfile(self.local_cfg):
            self.cfg = self.load()
        return {}

    def get_config_with_check_result(self) -> dict:
        try:
            self.check_result = CheckResultHandler(CHECK_FILE)
            self.servers = self.check_result.servers
            cfg = self.load()
            cfg[self.name]["lb"] = self.check_result.lb
            cfg[self.name]["deploy_type"] = "single" if len(self.check_result.config.servers) <= 1 else "cluster"
            cfg[self.name]["nginx_harbor_proxy_port"] = get_port("harbor", "nginx_harbor_proxy_port")
            cfg[self.name]["harbor_http_port"] = get_port("harbor", "http_port")
            cfg[self.name]["harbor_ipaddress"] = self.check_result.lb
            cfg[self.name]["master_harbor_ipaddress"] = self.servers.get_role_obj("harbor")[0].ipaddress
            return cfg
        except UtilsError:
            return {}

    def get_all_config_with_check_result(self) -> dict:
        try:
            cfg_dirs = path_join(self.laipvt_base_dir, "middleware")
            self.all_cfg = YamlConfig(cfg_dirs).read_dir()
        except Exception as e:
            print(e)
        return self.all_cfg

    def get_ocr_module_port(self, name: str) -> int:
        if name.endswith("_gpu"):
            n = name
        else:
            n = "{}_gpu".format(name)
        return get_port("ocr", n)


class MinioConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(MinioConfigHandler, self).__init__("minio")
        self.permit_keys = ("username", "password", "port", "nginx_proxy_port")
        self.cfg["username"] = "laiyelaiye"
        self.cfg["password"] = gen_pass()
        self.cfg["port"] = get_port("minio", "port")
        self.cfg["nginx_proxy_port"] = get_port("minio", "nginx_proxy_port")


class HarborConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(HarborConfigHandler, self).__init__("harbor")
        self.permit_keys = ("http_port", "password", "nginx_harbor_proxy_port")
        self.cfg["http_port"] = get_port("harbor", "http_port")
        self.cfg["nginx_harbor_proxy_port"] = get_port("harbor", "nginx_harbor_proxy_port")
        self.cfg["password"] = gen_pass()

class RedisConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(RedisConfigHandler, self).__init__("redis")
        self.permit_keys = ("password", "port", "port_sentinel", "master_name", "master_ip", "isslave")
        self.cfg["password"] = gen_pass()
        self.cfg["port_sentinel"] = get_port("redis", "port_sentinel")
        self.cfg["port"] = get_port("redis", "port")
        self.cfg["master_name"] = "mymaster"

class NginxConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(NginxConfigHandler, self).__init__("nginx")
        self.permit_keys = ("commander_port", "commander_tenant_port", "k8s_proxy_port", "mysql_proxy_port", "minio_proxy_port", "identity_proxy_port", "mage_proxy_port")
        self.cfg["commander_port"] = get_port("nginx", "commander_port")
        self.cfg["commander_tenant_port"] = get_port("nginx", "commander_tenant_port")
        self.cfg["k8s_proxy_port"] = get_port("nginx", "k8s_proxy_port")
        self.cfg["mage_proxy_port"] = get_port("nginx", "mage_proxy_port")


class KeepalivedConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(KeepalivedConfigHandler, self).__init__("keepalived")
        self.permit_keys = ("ipaddress", "master_network", "slave_network", "slave2_network")

class MysqlConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(MysqlConfigHandler, self).__init__("mysql")
        self.permit_keys = ("username", "password", "ipaddress", "port", "nginx_proxy_port", "lb", "proxysql_port")
        self.cfg["username"] = "root"
        self.cfg["password"] = gen_pass()
        self.cfg["port"] = get_port("mysql", "port")
        self.cfg["proxysql_port"] = get_port("mysql", "proxysql_port")
        self.cfg["nginx_proxy_port"] = get_port("mysql", "nginx_proxy_port")

class EsConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(EsConfigHandler, self).__init__("elasticsearch")
        self.permit_keys = ("username", "password", "ipaddress", "port", "http_port", "host_name", "master_node", "ip")
        self.cfg["http_port"] = get_port("elasticsearch", "http_port")
        self.cfg["tcp_port"] = get_port("elasticsearch", "tcp_port")
        self.cfg["username"] = "admin"
        self.cfg["password"] = gen_pass()

class RabbitmqConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(RabbitmqConfigHandler, self).__init__("rabbitmq")
        self.permit_keys = ("username", "password", "ipaddress", "port", "NODENAME", "HOSTNAME", "is_standalone")
        self.cfg["username"] = "admin"
        self.cfg["password"] = gen_pass()
        self.cfg["port"] = get_port("rabbitmq", "port")
        self.cfg["manage_port"] = get_port("rabbitmq", "manage_port")
        self.cfg["empd_port"] = get_port("rabbitmq", "empd_port")
        self.cfg["erlang_port"] = get_port("rabbitmq", "erlang_port")

class EtcdConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(EtcdConfigHandler, self).__init__("etcd")
        self.permit_keys = ("username", "password", "ipaddress", "http_port", "tcp_port")
        self.cfg["http_port"] = get_port("etcd", "http_port")
        self.cfg["tcp_port"] = get_port("etcd", "tcp_port")

class MonitorConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(MonitorConfigHandler, self).__init__("monitor")
        self.permit_keys = ("ipaddress", "grafana_port", "prometheus_port", "mysql_exporter_port", "redis_exporter_port",
                            "rabbitmq_exporter_port", "elasticsearch_exporter_port", "node_exporter_port"
                            "k8s_metrics_port", "")
        self.cfg["grafana_port"] = 3000
        self.cfg["prometheus_port"] = 9090
        self.cfg["mysql_exporter_port"] = 9104
        self.cfg["redis_exporter_port"] = 9121
        self.cfg["rabbitmq_exporter_port"] = 9419
        self.cfg["elasticsearch_exporter_port"] = 9114
        self.cfg["node_exporter_port"] = 9100
        self.cfg["k8s_metrics_port"] = 31388
        self.cfg["istio_prometheus_port"] = 31390

class SiberConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(SiberConfigHandler, self).__init__("siber")
        self.permit_keys = ("ipaddress", "port")
        self.cfg["port"] = 88

class IdentityConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(IdentityConfigHandler, self).__init__("identity")
        self.permit_keys = ("ipaddress", "port", "nginx_proxy_port")
        self.cfg["port"] = get_port("identity", "port")
        self.cfg["nginx_proxy_port"] = get_port("identity", "nginx_proxy_port")

class OcrConfigHandler(MiddlewareConfigHandler):
    def __init__(self):
        super(OcrConfigHandler, self).__init__("ocr")
        self.permit_keys = ("ocr_document_gpu", "ocr_table_gpu", "ocr_receipt_gpu",
                            "ocr_idcard_gpu", "ocr_bankcard_gpu" "ocr_vehicle_gpu",
                            "ocr_vehiclelicense_gpu", "ocr_biz_gpu", "passport_gpu")
        self.cfg["ocr_document_gpu"] = get_port("ocr", "ocr_document_gpu")
        self.cfg["ocr_table_gpu"] = get_port("ocr", "ocr_table_gpu")
        self.cfg["ocr_receipt_gpu"] = get_port("ocr", "ocr_receipt_gpu")
        self.cfg["ocr_idcard_gpu"] = get_port("ocr", "ocr_idcard_gpu")
        self.cfg["ocr_bankcard_gpu"] = get_port("ocr", "ocr_bankcard_gpu")
        self.cfg["ocr_vehicle_gpu"] = get_port("ocr", "ocr_vehicle_gpu")
        self.cfg["ocr_vehiclelicense_gpu"] = get_port("ocr", "ocr_vehiclelicense_gpu")
        self.cfg["ocr_biz_gpu"] = get_port("ocr", "ocr_biz_gpu")
        self.cfg["ocr_passport_gpu"] = get_port("ocr", "ocr_passport_gpu")

    def get_ocr_module_port(self, module_name: str) -> int:
        return get_port("ocr", module_name)



from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.sysutil.policy import is_port, is_ip_or_fqdn, is_password
from laipvt.helper.exception import LaiyeException

def check_value(fn):
    def wrapper(self, *args, **kwargs):
        res = True
        if "port" in args[0] or "ocr_" in args[0]:
            if not is_port(args[1]): raise LaiyeException("参数{}的类型不正确, 端口范围: 0-65535".format(args[0]))
        elif "ip" in args[0]:
            if len(args[1]) == 0:
                res = True
            else:
                if isinstance(args[1], list):
                    for ip in args[1]:
                        if not is_ip_or_fqdn(ip): raise LaiyeException("参数{}的类型不正确, 非法的ip地址".format(args[0]))
                else:
                    if not is_ip_or_fqdn(args[1]): raise LaiyeException("参数{}的类型不正确, 非法的ip地址".format(args[0]))
        elif "passwo" in args[0]:
            if not is_password(args[1]): raise LaiyeException("参数{}的类型不正确, 密码复杂度不匹配，要求长度大于8位，并且至少包含一个数字，小写字母，大写字母".format(args[0]))
        if res:
            fn(self, *args, **kwargs)
        else:
            raise LaiyeException("参数{}的类型不正确".format(args[0]))
    return wrapper